@@ -29,5 +29,5 @@ sortBy order names df
       -- TODO: Remove the SortOrder defintion from operations so we can share it between here and internal and
       -- we don't have to do this Bool mapping.
       indexes = sortedIndexes' (order == Ascending) (toRowVector names df)
-      pick idxs col = atIndicesStable idxs <$> col
+      pick idxs col = atIndicesStable idxs col
     in df {columns = V.map (pick indexes) (columns df)}
