@@ -4,8 +4,8 @@
 {-# LANGUAGE ExplicitNamespaces #-}
 {-# LANGUAGE GADTs #-}
 {-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE StrictData #-}
 {-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE BangPatterns #-}
 module DataFrame.Operations.Statistics where
 
 import qualified Data.List as L
@@ -43,8 +43,8 @@ frequencies name df = case getColumn name df of
         Nothing -> case testEquality (typeRep @a) (typeRep @String) of
           Just Refl -> T.pack c'
           Nothing -> (T.pack . show) c'
-      initDf = empty & insertColumn "Statistic" (V.fromList ["Count" :: T.Text,  "Percentage (%)"])
-    in L.foldl' (\df (col, k) -> insertColumn (vText col) (V.fromList [k, k * 100 `div` total]) df) initDf counts
+      initDf = empty & insertVector "Statistic" (V.fromList ["Count" :: T.Text,  "Percentage (%)"])
+    in L.foldl' (\df (col, k) -> insertVector (vText col) (V.fromList [k, k * 100 `div` total]) df) initDf counts
   Just ((OptionalColumn (column :: V.Vector a))) -> let
       counts = valueCounts @a name df
       total = P.sum $ map snd counts
@@ -54,8 +54,8 @@ frequencies name df = case getColumn name df of
         Nothing -> case testEquality (typeRep @a) (typeRep @String) of
           Just Refl -> T.pack c'
           Nothing -> (T.pack . show) c'
-      initDf = empty & insertColumn "Statistic" (V.fromList ["Count" :: T.Text,  "Percentage (%)"])
-    in L.foldl' (\df (col, k) -> insertColumn (vText col) (V.fromList [k, k * 100 `div` total]) df) initDf counts
+      initDf = empty & insertVector "Statistic" (V.fromList ["Count" :: T.Text,  "Percentage (%)"])
+    in L.foldl' (\df (col, k) -> insertVector (vText col) (V.fromList [k, k * 100 `div` total]) df) initDf counts
   Just ((UnboxedColumn (column :: VU.Vector a))) -> let
       counts = valueCounts @a name df
       total = P.sum $ map snd counts
@@ -65,11 +65,12 @@ frequencies name df = case getColumn name df of
         Nothing -> case testEquality (typeRep @a) (typeRep @String) of
           Just Refl -> T.pack c'
           Nothing -> (T.pack . show) c'
-      initDf = empty & insertColumn "Statistic" (V.fromList ["Count" :: T.Text,  "Percentage (%)"])
-    in L.foldl' (\df (col, k) -> insertColumn (vText col) (V.fromList [k, k * 100 `div` total]) df) initDf counts
+      initDf = empty & insertVector "Statistic" (V.fromList ["Count" :: T.Text,  "Percentage (%)"])
+    in L.foldl' (\df (col, k) -> insertVector (vText col) (V.fromList [k, k * 100 `div` total]) df) initDf counts
+  _ -> error $ "There are ungrouped columns"
 
 mean :: T.Text -> DataFrame -> Maybe Double
-mean = applyStatistic SS.mean
+mean = applyStatistic mean'
 
 median :: T.Text -> DataFrame -> Maybe Double
 median = applyStatistic (SS.median SS.medianUnbiased)
@@ -81,7 +82,7 @@ skewness :: T.Text -> DataFrame -> Maybe Double
 skewness = applyStatistic SS.skewness
 
 variance :: T.Text -> DataFrame -> Maybe Double
-variance = applyStatistic SS.variance
+variance = applyStatistic variance'
 
 interQuartileRange :: T.Text -> DataFrame -> Maybe Double
 interQuartileRange = applyStatistic (SS.midspread SS.medianUnbiased 4)
@@ -101,26 +102,25 @@ _getColumnAsDouble name df = case getColumn name df of
       Nothing -> Nothing
   _ -> Nothing
 
-sum :: T.Text -> DataFrame -> Maybe Double
+sum :: forall a. (Columnable a, Num a, VU.Unbox a) => T.Text -> DataFrame -> Maybe a
 sum name df = case getColumn name df of
   Nothing -> throw $ ColumnNotFoundException name "sum" (map fst $ M.toList $ columnIndices df)
-  Just ((UnboxedColumn (column :: VU.Vector a'))) -> case testEquality (typeRep @a') (typeRep @Int) of
-    Just Refl -> Just $ VG.sum (VU.map fromIntegral column)
-    Nothing -> case testEquality (typeRep @a') (typeRep @Double) of
-      Just Refl -> Just $ VG.sum column
-      Nothing -> Nothing
+  Just ((UnboxedColumn (column :: VU.Vector a'))) -> case testEquality (typeRep @a') (typeRep @a) of
+    Just Refl -> Just $ VG.sum column
+    Nothing -> Nothing
 
 applyStatistic :: (VU.Vector Double -> Double) -> T.Text -> DataFrame -> Maybe Double
-applyStatistic f name df = do
-      column <- getColumn name df
-      if columnTypeString column == "Double"
-      then reduceColumn f column
-      else do
-        matching <- asum [mapColumn (fromIntegral :: Int -> Double) column,
-                          mapColumn (fromIntegral :: Integer -> Double) column,
-                          mapColumn (realToFrac :: Float -> Double) column,
-                          Just column ]
-        reduceColumn f matching
+applyStatistic f name df = case getColumn name df of
+      Nothing -> throw $ ColumnNotFoundException name "applyStatistic" (map fst $ M.toList $ columnIndices df)
+      Just column@(UnboxedColumn (col :: VU.Vector a)) -> case testEquality (typeRep @a) (typeRep @Double) of
+        Just Refl -> reduceColumn f column
+        Nothing -> do
+          matching <- asum [mapColumn (fromIntegral :: Int -> Double) column,
+                mapColumn (fromIntegral :: Integer -> Double) column,
+                mapColumn (realToFrac :: Float -> Double) column,
+                Just column ]
+          reduceColumn f matching
+      _ -> Nothing
 
 applyStatistics :: (VU.Vector Double -> VU.Vector Double) -> T.Text -> DataFrame -> Maybe (VU.Vector Double)
 applyStatistics f name df = case getColumn name df of
@@ -135,7 +135,7 @@ applyStatistics f name df = case getColumn name df of
 
 summarize :: DataFrame -> DataFrame
 summarize df = fold columnStats (columnNames df) (fromNamedColumns [("Statistic", fromList ["Mean" :: T.Text, "Minimum", "25%" ,"Median", "75%", "Max", "StdDev", "IQR", "Skewness"])])
-  where columnStats name d = if all isJust (stats name) then insertUnboxedColumn name (VU.fromList (map (roundTo 2 . fromMaybe 0) $ stats name)) d else d
+  where columnStats name d = if all isJust (stats name) then insertUnboxedVector name (VU.fromList (map (roundTo 2 . fromMaybe 0) $ stats name)) d else d
         stats name = let
             quantiles = applyStatistics (SS.quantilesVec SS.medianUnbiased (VU.fromList [0,1,2,3,4]) 4) name df
             min' = flip (VG.!) 0 <$> quantiles
@@ -155,3 +155,27 @@ summarize df = fold columnStats (columnNames df) (fromNamedColumns [("Statistic"
               skewness name df]
         roundTo :: Int -> Double -> Double
         roundTo n x = fromInteger (round $ x * (10^n)) / (10.0^^n)
+
+mean' :: VU.Vector Double -> Double
+mean' samp = let
+    (!total, !n) = VG.foldl' (\(!total, !n) v -> (total + v, n + 1))  (0 :: Double, 0 :: Int) samp
+  in total / fromIntegral n
+
+-- accumulator: count, mean, m2
+data VarAcc = VarAcc !Int !Double !Double  deriving Show
+
+step :: VarAcc -> Double -> VarAcc
+step (VarAcc !n !mean !m2) !x =
+  let !n'    = n + 1
+      !delta = x - mean
+      !mean' = mean + delta / fromIntegral n'
+      !m2'   = m2 + delta * (x - mean')
+  in  VarAcc n' mean' m2'
+
+computeVariance :: VarAcc -> Double
+computeVariance (VarAcc n _ m2)
+  | n < 2     = 0                -- or error "variance of <2 samples"
+  | otherwise = m2 / fromIntegral (n - 1)
+
+variance' :: VU.Vector Double -> Double
+variance' = computeVariance . VG.foldl' step (VarAcc 0 0 0)
